@@ -4,6 +4,9 @@ represents a Carbon order book position
 (c) Copyright Bprotocol foundation 2022.
 Licensed under MIT
 """
+__version__ = "1.0"
+__date__ = "8/Dec/2022"
+
 import pprint
 from dataclasses import dataclass
 from decimal import Decimal
@@ -27,19 +30,24 @@ class Order:
     Curve class to represent a curve on the market maker's order book
     """
 
+    __VERSION__ = __version__
+    __DATE__    = __date__
+
     # Production variables
-    B: DecFloatInt = None  # = sqrt(price_low)
-    S: DecFloatInt = None  # = sqrt(price_high) - sqrt(price_low)
-    y_int: DecFloatInt = None  # is the initial intercept of the curve
+    B: DecFloatInt = None       # = sqrt(price_low)
+    S: DecFloatInt = None       # = sqrt(price_high) - sqrt(price_low)
+    y_int: DecFloatInt = None   # the initial intercept of the curve
 
     # Original variables
-    p_high: DecFloatInt = None  # = high price, alias "p_a" in original code/docs
-    p_low: DecFloatInt = None  # = low price, alias "p_a" in original code/docs
-    _y: DecFloatInt = None  # is the current balance of the curve
+    p_high: DecFloatInt = None  # = high price, also p_a, p_start
+    p_low: DecFloatInt = None   # = low price, also p_b, p_low
+    p_marginal: DecFloatInt = None  # = marginal price, also p_c, p_mid
+    disabled: bool = False      # if True, prices used to be None
+    _y: DecFloatInt = None      # the current balance of the curve
 
     # Alternate variables
-    D: DecFloatInt = None  # is the current intercept of the curve
-    _C: DecFloatInt = None  # is the current balance of the curve
+    D: DecFloatInt = None       # the current intercept of the curve
+    _C: DecFloatInt = None      # the current balance of the curve
 
     pair: CarbonPair = None
     tkn: str = None
@@ -140,8 +148,13 @@ class Order:
         if self.p_high is not None and self.p_low is not None:
             self.p_high = Decimal(self.p_high)
             self.p_low = Decimal(self.p_low)
+
+        if self.y_int is not None:
             self.y_int = Decimal(self.y_int)
 
+        if self.p_marginal is not None:
+            self.p_marginal = Decimal(self.p_marginal)
+
         if self.auto_convert_variables:
 
             # Init original variables if alternate variables are set
@@ -173,8 +186,16 @@ class Order:
             if self.y_int is not None:
                 self.D = self.C = self._y = self.y_int
 
+            elif self.y_int is None and self.p_marginal is not None:
+                if type(self.p_marginal) is Decimal:
+                    self.y_int = self._y * (self.p_high.sqrt() - self.p_low.sqrt()) / (self.p_marginal.sqrt() - self.p_low.sqrt())
+                else:
+                    self.y_int = float(self._y) * (sqrt(self.p_high) - sqrt(self.p_low)) / (sqrt(self.p_marginal) - sqrt(self.p_low))
+
             elif self.D is not None:
                 self.y_int = self.C = self._y = self.D
 
             if self.p_high == self.p_low:
                 self.adj_n_mode = True
+
+
