@@ -7,9 +7,12 @@ Licensed under MIT
 VERSION HISTORY
 v2.0 - require slashpair notation (breaking change); exclude_future, constants for matching method
 v2.2 - curve disabling, single orders
+v2.2.1 - CarbonOrderUI linked
+v2.3 - added fast router
+v2.4 - limit orders, changed trade_action -> match_by in _trade
 """
-__version__ = "2.2"
-__date__ = "8/Dec/2022"
+__version__ = "2.4"
+__date__ = "12/Jan/2023"
 
 import itertools
 from typing import Callable, Any, Tuple, Dict, List
@@ -19,12 +22,13 @@ from tabulate import tabulate
 from ..order import Order
 from ..pair import CarbonPair
 from ..carbon_order_ui import CarbonOrderUI
-from ..routers import ExactRouterX0Y0N, AlphaRouter
+from ..routers import ExactRouterX0Y0N, AlphaRouter, FastRouter
 from decimal import Decimal
 import pandas as pd
 import numpy as np
 
 
+
 class CarbonSimulatorUI:
     """
     Providing the user interface for the Carbon simulation
@@ -44,15 +48,15 @@ class CarbonSimulatorUI:
     __DATE__    = __date__
 
     MATCH_EXACT = "exact"
-    MATCH_FAST = "fast_"
-    MATCH_ALPHA = "alpha_"
+    MATCH_FAST = "fast"
+    MATCH_ALPHA = "alpha"
 
     def __init__(
             self,
             verbose: bool = False,
             pair: Any = None,
             raiseonerror: bool = False,
-            decimals: int = 6,
+            decimals: int = 50,
             matching_method: str = MATCH_EXACT,
             exclude_future: bool = True,
     ):
@@ -71,6 +75,8 @@ class CarbonSimulatorUI:
             self.matcher = AlphaRouter(verbose=False)
         elif matching_method == self.MATCH_EXACT:
             self.matcher = ExactRouterX0Y0N(verbose=False)
+        elif matching_method == self.MATCH_FAST:
+            self.matcher = FastRouter(verbose=False)
         elif matching_method == self.MATCH_FAST:
             raise ValueError("Fast router not implemented", matching_method)
         else:
@@ -166,6 +172,7 @@ class CarbonSimulatorUI:
             carbon_pair: CarbonPair,
             id1: int = None,
             id2: int = None,
+            p_marginal: Any = None,
     ) -> int:
         """
         PRIVATE - adds a position for sale of tkn
@@ -176,6 +183,7 @@ class CarbonSimulatorUI:
         :p_hi:          ditto upper; also p_a, p_start
         :carbon_pair:   the token pair a CarbonPair object
         :id1/2:         the order ids of the two orders that make up the position; if not given, they are generated
+        :p_marginal:    the marginal price of the position; if not given, it is calculated
         :returns:       the id of the order added
         """
 
@@ -190,7 +198,7 @@ class CarbonSimulatorUI:
 
         if not (p_lo is None and p_hi is None):
 
-            # check the they are both numbers
+            # check that they are both numbers
             if p_lo is None or p_hi is None:
                 raise ValueError("p_lo, p_hi must either be both None or both numbers", p_lo, p_hi)
 
@@ -210,17 +218,21 @@ class CarbonSimulatorUI:
 
         # enter order
         order_params = {
-            "tkn": tkn,                                 # the token being sold
-            "y_int": amt,                               # the capacity of the curve
-            "_y": amt,                                  # the initial holding is of the curve (in `tkn`)
-            "p_low": p_lo,                              # the lower end of the range (`tkn` numeraire); also p_b, p_end
-            "p_high": p_hi,                             # the upper end of the range (`tkn` numeraire); also p_a, p_start
-            "disabled": disabled,                       # if True, order is disabled (p=0)
-            "pair_name": carbon_pair.pair_iso,          # the iso name of the pair
-            "pair": carbon_pair,                        # the CarbonPair object
-            "id": id1,                                  # the id of the new curve generated
-            "linked_to_id": id2,                        # the id to which this curve is linked (=id if single curve)
+            "tkn": tkn,                                     # the token being sold
+            "_y": amt,                                       # the marginal price of the curve
+            "p_low": p_lo,                                  # the lower end of the range (`tkn` numeraire); also p_b, p_end
+            "p_high": p_hi,                                 # the upper end of the range (`tkn` numeraire); also p_a, p_start
+            "disabled": disabled,                           # if True, order is disabled (p=0)
+            "pair_name": carbon_pair.pair_iso,              # the iso name of the pair
+            "pair": carbon_pair,                            # the CarbonPair object
+            "id": id1,                                      # the id of the new curve generated
+            "linked_to_id": id2,                            # the id to which this curve is linked (=id if single curve)
         }
+        if p_marginal is not None:
+            order_params["p_marginal"] = p_marginal         # the marginal price of the curve
+
+        if p_marginal is None:
+            order_params["y_int"] = amt                     # the capacity of the curve
 
         self.orders[id1] = Order(**order_params)
         return id1
@@ -323,6 +335,8 @@ class CarbonSimulatorUI:
             pbuy_start: Any = None,
             pbuy_end: Any = None,
             pair: str = None,
+            psell_marginal: Any = None,
+            pbuy_marginal: Any = None,
     ) -> Dict[str, Any]:
         """
         adds two linked orders (one buy, one sell; aka a "strategy")
@@ -335,6 +349,8 @@ class CarbonSimulatorUI:
         :pbuy_start:    start of the of the buy `tkn` range*, quoted in the price convention of `pair`
         :pbuy_end:      ditto end
         :pair:          the token pair to which the strategy corresponds, eg "ETHUSD"
+        :pbuy_marginal: the current price of the other token in the pair
+        :psell_marginal: the current price of `tkn` in the pair
 
         *px_start, px_end are interchangeable, the code deals with sorting them, albeit issuing a warning
         message if they are in the wrong order; sell and buy is seen from the perspective of the AMM, which
@@ -352,30 +368,31 @@ class CarbonSimulatorUI:
             if tkn2 is None:
                 raise ValueError("Token not in pair", tkn, carbon_pair.slashpair, pair.slashpair)
 
-
             # create order tracking ids
             id1 = self._posid
             id2 = self._posid
 
             # convert the prices from the pair numeraire to the curve numeraire
+            psell_marginal_c = carbon_pair.convert_price(psell_marginal, tkn)
             psell_start_c = carbon_pair.convert_price(psell_start, tkn)
             psell_end_c = carbon_pair.convert_price(psell_end, tkn)
             pbuy_start_c = carbon_pair.convert_price(pbuy_start, tkn2)
             pbuy_end_c = carbon_pair.convert_price(pbuy_end, tkn2)
+            pbuy_marginal_c = carbon_pair.convert_price(pbuy_marginal, tkn2)
 
             # ugly hack but the 1/2 range boundaries are in the wrong order
             # we want the closer boundary first
             self._add_order_sell_tkn(
-                tkn, amt_sell, psell_end_c, psell_start_c, carbon_pair, id1, id2
+                tkn, amt_sell, psell_end_c, psell_start_c, carbon_pair, id1, id2, psell_marginal_c
             )
             self._add_order_sell_tkn(
-                tkn2, amt_buy, pbuy_end_c, pbuy_start_c, carbon_pair, id2, id1
+                tkn2, amt_buy, pbuy_end_c, pbuy_start_c, carbon_pair, id2, id1, pbuy_marginal_c
             )
 
             if self.verbose:
                 print(
-                    f"[add_linked_pos] added tkn={tkn}, amt_sell={amt_sell}, psell_start={psell_start}, "
-                    f"psell_end={psell_end}, amt_buy={amt_buy}, pbuy_start={pbuy_start}, pbuy_end={pbuy_end}, "
+                    f"[add_linked_pos] added tkn={tkn}, amt_sell={amt_sell}, psell_start={psell_start}, psell_marginal_c={psell_marginal_c}, "
+                    f"psell_end={psell_end}, amt_buy={amt_buy}, pbuy_start={pbuy_start}, pbuy_end={pbuy_end}, pbuy_marginal_c={pbuy_marginal_c},"
                     f"pair={carbon_pair.pair_iso}"
                 )
 
@@ -440,19 +457,26 @@ class CarbonSimulatorUI:
     #delete_pos = delete_order
     delete_strategy = delete_order
 
+    MATCH_BY_SOURCE = "match_by_source"
+    MATCH_BY_TARGET = "match_by_target"
+
     def _trade(
             self,
             tkn: str,
             amt: Any,
+            support_partial: bool,
             carbon_pair: CarbonPair,
-            trade_action: Callable = None,
+            #trade_action: Callable = None,
+            match_by: str = None,
             trade_description: str = None,
             execute: bool = True,
             limit_price: Any = None,
+            limit_amt: Any = None,
             inpair: bool = True,
             use_positions: List[int] = None,
             threshold_orders: int = None,
             use_positions_matchlevel: List[int] = [],
+            is_by_target: bool = False,
 
     ) -> Dict[str, Any]:
         """
@@ -461,19 +485,47 @@ class CarbonSimulatorUI:
         :tkn:                   the token that is being SOLD by the AMM, eg "ETH"*
         :amt:                   the amount to be traded*
         :carbon_pair:           the CarbonPair class of the token pair
-        :trade_action:          either `match_by_target(...)` or `match_by_src(...)` method of the `self.matcher`
+        :match_by:              either MATCH_BY_SOURCE or MATCH_BY_TARGET
         :trade_description:     human-readable description of the trade, eg "amm vs trader, buy vs sell, for amt"
         :execute:               if True (default), the trade is executed; otherwise only routing is shown
         :limit_price:           the limit price of the order (this price or better from point of view
-                                of the trader, not the AMM!), quoted in convention of the pair
+                                of the trader, not the AMM!), quoted in convention of the pair**
+        :limit_amt:             the minimum amount of tokens the traders expects to obtain; quoted in units of the
+                                other token, ie `carbon_pair.other(tkn)`**
         :inpair:                if True, only match within pair; if False (default), route through all available pairs
         :use_positions:         the positions to use for the trade (default: all positions)
         :threshold_orders:      the maximum number of order to be routed through using the alpha router
+        :support_partial:       if True (not default), and insufficient liquidity for a trade request, a partial fullfilment is made
 
-        *amt is always effectively a positive amount; however, if `trade_action` is `match_by_target` then it must
-        be provided as a negative number, and if `match_by_src` as positive number
+        *amt is always effectively a positive amount; however, if `match_by` is `MATCH_BY_TARGET` then it must
+        be provided as a negative number, and if `MATCH_BY_SOURCE` as positive number
+
+        **limit_price and limit_amt are redundant; either one of them can be given, or none, but not both
         """
         trades, orders = None, None
+        is_partial = False
+
+        if match_by == self.MATCH_BY_TARGET:
+            trade_action = self.matcher.match_by_target
+        elif match_by == self.MATCH_BY_SOURCE:
+            trade_action = self.matcher.match_by_src
+        else:
+            raise ValueError("match_by must be self.MATCH_BY_TARGET or self.MATCH_BY_SOURCE", match_by)
+
+        if not limit_amt is None:
+            if not limit_price is None:
+                raise ValueError("Not both limit_amt and limit_price can be given", limit_amt, limit_price)
+            
+            # amt is expressed in the source token if match_by == MATCH_BY_SOURCE, otherwise target token
+            # rawprice is expressed as tkn per other token if MATCH_BY_TARGET, otherwise reverse
+            # we use carbon_pair.convert_price to convert it into the proper price convention
+            rawprice = abs(float(amt) / float(limit_amt))
+            limit_price = carbon_pair.convert_price(
+                rawprice, tknq = tkn if match_by == self.MATCH_BY_TARGET else carbon_pair.other(tkn)
+            )
+            #print(f"[_trade] limit_amt={limit_amt}, tkn={tkn}, match_by={match_by}, limit_price={limit_price}")
+            #print(f"[_trade] limit_amt={limit_amt}, tkn={tkn}, rawprice={rawprice}, limit_price={limit_price}")
+            
         try:
             decimals = self.decimals
             carbon_pair_r = carbon_pair.reverse
@@ -508,7 +560,8 @@ class CarbonSimulatorUI:
             # route the trade
             routes = trade_action(
                 x=amt,
-                threshold_orders=threshold_orders
+                threshold_orders=threshold_orders,
+                support_partial=support_partial,
             )
 
             # retrieve the final route
@@ -541,7 +594,7 @@ class CarbonSimulatorUI:
             # in other words: `total_output` is measured in `tkn` and `total_input` in `otkn`
             # therefore, the numeraire of raw_p = out/in is the token associated to out, `tkn`
             price_avg = carbon_pair.convert_price(raw_price, tkn)
-            price_avg = round(float(price_avg), decimals)
+            # price_avg = round(float(price_avg), decimals)
             if self.debug:
                 print("[_trade] final routes", routes)
 
@@ -563,8 +616,9 @@ class CarbonSimulatorUI:
             # - `buysell = BUY`
 
             if limit_price is not None:
+                print(f"[_trade] limit_price={limit_price}, price_avg={price_avg}")
                 limitfail = not carbon_pair.limit_is_met(
-                    tkn, limit_price, carbon_pair.BUY, price_avg
+                    tkn, limit_price, carbon_pair.BUY, round(float(price_avg), decimals)
                 )
                 if limitfail:
                     execute = False
@@ -597,9 +651,9 @@ class CarbonSimulatorUI:
                 )
 
                 if execute:
-                    self.orders[indx].y -= Decimal(amt1)
+                    self.orders[indx].y -= Decimal(amt1) if not (is_by_target and isinstance(self.matcher, FastRouter)) else Decimal(amt2)
                     if self.orders[indx].linked_to_id != indx:
-                        self.orders[self.orders[indx].linked_to_id].y += Decimal(amt2)
+                        self.orders[self.orders[indx].linked_to_id].y += Decimal(amt2) if not (is_by_target and isinstance(self.matcher, FastRouter)) else Decimal(amt1)
                         if (
                                 self.orders[self.orders[indx].linked_to_id].y
                                 > self.orders[self.orders[indx].linked_to_id].y_int
@@ -621,12 +675,19 @@ class CarbonSimulatorUI:
                     "exec": [
                         execute
                     ],  # True iff this trade has been executed (also False on limitfail)
+                    "partial": [
+                        is_partial
+                    ],  # True support_partial is set to True
                     "limitfail": [
                         limitfail
                     ],  # False if limit met, True if not, None if no limit
-                    "amt1": [amt1],  # the amount of `tkn1` being sold by the AMM (>0)
+                    "amt1": [
+                        amt1 if not (is_by_target and isinstance(self.matcher, FastRouter)) else amt2
+                        ],  # the amount of `tkn1` being sold by the AMM (>0)
                     "tkn1": [tkn],  # the token being SOLD by the AMM
-                    "amt2": [amt2],  # the amount of `tkn2` being bought by the AMM (>0)
+                    "amt2": [
+                        amt2 if not (is_by_target and isinstance(self.matcher, FastRouter)) else amt1
+                        ],  # the amount of `tkn2` being bought by the AMM (>0)
                     "tkn2": [tkno],  #  the token being BOUGHT by the AMM
                     "pair": [
                         carbon_pair.pair_iso
@@ -638,7 +699,7 @@ class CarbonSimulatorUI:
                         1
                     ],  # the number of routes across which the trade was executed
                     "price": [
-                        price_avg
+                        round(float(price_avg), decimals) if not (is_by_target and isinstance(self.matcher, FastRouter)) else round(float(1 / Decimal(price_avg)), decimals)
                     ],  # the price amt_/amt_, in the convention of the pair
                     "p_unit": [
                         f"{tknq} per {tknb}"
@@ -658,11 +719,13 @@ class CarbonSimulatorUI:
             # Handle base token vs quote token variable swaps for the trade
             ttl_input = round(abs(routes[-1].total_input), decimals)
             ttl_output = round(abs(routes[-1].total_output), decimals)
-            price_avg = f"{price_avg}"
+            # price_avg = f"{price_avg}"
 
             amt1, amt2 = ttl_output, ttl_input
+            if amt != ttl_input:
+                is_partial = True
 
-            note = f"AMM sells {amt1:.0f}{tkn} buys {amt2:.0f}{tkno}"
+            note = f"AMM sells {amt1 if not (is_by_target and isinstance(self.matcher, FastRouter)) else amt2:.0f}{tkn} buys {amt2 if not (is_by_target and isinstance(self.matcher, FastRouter)) else amt1:.0f}{tkno}"
             num_trades = len(routes)
             order_ids = [id_map[o.index] for o in routes]
             uid = f"{numtrades}"
@@ -674,15 +737,22 @@ class CarbonSimulatorUI:
                 "note": [note],
                 "aggr": [True],
                 "exec": [execute],
+                "partial": [is_partial],
                 "limitfail": [limitfail],
-                "amt1": [amt1],
+                "amt1": [
+                    amt1 if not (is_by_target and isinstance(self.matcher, FastRouter)) else amt2
+                    ],
                 "tkn1": [tkn],
-                "amt2": [amt2],
+                "amt2": [
+                    amt2 if not (is_by_target and isinstance(self.matcher, FastRouter)) else amt1
+                    ],
                 "tkn2": [tkno],
                 "pair": [carbon_pair.pair_iso],
                 "routeix": [str(order_ids)],
                 "nroutes": [num_trades],
-                "price": [price_avg],
+                "price": [
+                    round(float(price_avg), decimals) if not (is_by_target and isinstance(self.matcher, FastRouter)) else round(float(1 / Decimal(price_avg)), decimals)
+                    ],
                 "p_unit": [f"{tknq} per {tknb}"],
             }
 
@@ -704,7 +774,7 @@ class CarbonSimulatorUI:
             if self.raiseonerror:
                 raise
             return {"success": False, "error": str(e), "exception": e}
-        return {"success": True, "trades": trades}
+        return {"success": True, "trades": trades, "is_by_target": is_by_target, "tkn_sold": tkn}
 
     def amm_buys(
             self,
@@ -714,9 +784,11 @@ class CarbonSimulatorUI:
             execute: bool = True,
             inpair: bool = True,
             limit_price: Any = None,
+            limit_amt: Any = None,
             threshold_orders: int = 10,
             use_positions: List[int] = None,
             use_positions_matchlevel: List[int] = [],
+            support_partial: bool = False,
     ) -> Dict[str, Any]:
         """
         the AMM buys (and the trader sells) `amt` > 0 of `tkn`
@@ -727,11 +799,19 @@ class CarbonSimulatorUI:
         :execute:           if True (default), the trade is executed; otherwise only routing is shown
         :inpair:            if True, only match within pair; if False (default), route through all available pairs
         :limit_price:       the limit price of the order (this price or better from point of view
-                            of the trader, not the AMM!), quoted in convention of the pair
+                            of the trader, not the AMM!), quoted in convention of the pair*
+        :limit_amt:         the minimum amount of tokens the traders expects to obtain; quoted in units of the
+                            other token, ie `carbon_pair.other(tkn)`*
         :threshold_orders:  the maximum number of order to be routed through using the alpha router
-        :use_positions: the positions to use for the trade (default: all positions)
+        :use_positions:     the positions to use for the trade (default: all positions)
+        :support_partial:   if True (not default), and insufficient liquidity for a trade request, a partial fullfilment is made
+        
+        *limit_price and limit_amt are redundant; either one of them can be given, or none, but not both
         """
 
+        #print("[amm_buys]", limit_amt, limit_price)
+        
+
         try:
 
             if amt < 0:
@@ -743,26 +823,31 @@ class CarbonSimulatorUI:
                     execute=execute,
                     inpair=inpair,
                     limit_price=limit_price,
+                    limit_amt=limit_amt,
                     threshold_orders=threshold_orders,
+                    support_partial=support_partial,
                 )
 
             # get the token `tkn`, the other token `tkno` and the CarbonPair object
             tkn, tkno, carbon_pair = self._get_tkn_and_validate(tkn, pair)
             self._assert_position_is_valid(tkno, carbon_pair)
-
             return self._trade(
                 tkn=tkno,
                 amt=Decimal(str(amt)),
                 carbon_pair=carbon_pair,
-                trade_action=self.matcher.match_by_src,
+                #trade_action=self.matcher.match_by_src,
+                match_by = self.MATCH_BY_SOURCE,
                 trade_description=f"trader buys $ {tkno} sells * {tkn} | AMM sells $ {tkno} buys * {tkn} via "
                                   f"order #",
                 execute=execute,
                 limit_price=limit_price,
+                limit_amt=limit_amt,
                 inpair=inpair,
                 threshold_orders=threshold_orders,
                 use_positions=use_positions,
                 use_positions_matchlevel=use_positions_matchlevel,
+                is_by_target=False,
+                support_partial=support_partial,
             )
         except Exception as e:
             if self.raiseonerror:
@@ -779,9 +864,11 @@ class CarbonSimulatorUI:
             execute: bool = True,
             inpair: bool = True,
             limit_price: Any = None,
+            limit_amt: Any = None,
             threshold_orders: int = 10,
             use_positions: List[int] = None,
             use_positions_matchlevel: List[int] = [],
+            support_partial: bool = False,
     ) -> Dict[str, Any]:
         """
         the AMM sells (and the trader buys) `amt` > 0 of `tkn`
@@ -792,10 +879,18 @@ class CarbonSimulatorUI:
         :execute:           if True (default), the trade is executed; otherwise only routing is shown
         :inpair:            if True, only match within pair; if False (default), route through all available pairs
         :limit_price:       the limit price of the order (this price or better from point of view
-                            of the trader, not the AMM!), quoted in convention of the pair
+                            of the trader, not the AMM!), quoted in convention of the pair*
+        :limit_amt:         the minimum amount of tokens the traders expects to obtain; quoted in units of the
+                            other token, ie `carbon_pair.other(tkn)`*
         :threshold_orders:  the maximum number of order to be routed through using the alpha router 
-        :use_positions: the positions to use for the trade (default: all positions)
+        :use_positions:     the positions to use for the trade (default: all positions)
+        :support_partial:   if True (not default), and insufficient liquidity for a trade request, a partial fullfilment is made
+        
+        *limit_price and limit_amt are redundant; either one of them can be given, or none, but not both
         """
+
+        #print("[amm_sells]", limit_amt, limit_price)
+        
         try:
 
             if amt < 0:
@@ -807,7 +902,9 @@ class CarbonSimulatorUI:
                     execute=execute,
                     inpair=inpair,
                     limit_price=limit_price,
+                    limit_amt=limit_amt,
                     threshold_orders=threshold_orders,
+                    support_partial=support_partial,
                 )
 
             # get the token `tkn`, the other token `tkno` and the CarbonPair object
@@ -818,15 +915,19 @@ class CarbonSimulatorUI:
                 tkn=tkn,
                 amt=-Decimal(str(amt)),
                 carbon_pair=carbon_pair,
-                trade_action=self.matcher.match_by_target,
+                #trade_action=self.matcher.match_by_target,
+                match_by = self.MATCH_BY_TARGET,
                 trade_description=f"trader buys $ {tkno} sells * {tkn} | AMM sells $ {tkn} buys * {tkno} via "
                                   f"order #",
                 execute=execute,
                 limit_price=limit_price,
+                limit_amt=limit_amt,
                 inpair=inpair,
                 threshold_orders=threshold_orders,
                 use_positions=use_positions,
                 use_positions_matchlevel=use_positions_matchlevel,
+                is_by_target=True,
+                support_partial=support_partial,
 
             )
 
@@ -838,7 +939,7 @@ class CarbonSimulatorUI:
     trader_buys = amm_sells
 
     @staticmethod
-    def _to_pandas(order: Order, decimals: int = 6) -> pd.DataFrame:
+    def _to_pandas(order: Order, decimals: int = 50) -> pd.DataFrame:
         """
         Exports Order values for inspection...
         """
@@ -882,7 +983,13 @@ class CarbonSimulatorUI:
                         value, only state corresponding to this specific `pair` is provided
         :inclhistory:   if True (default), also provide history, in particular trade history If False, no trades
                         are provided
-        :returns:       a dict with the current state of the system where the values are pandas dataframes
+        :returns:       a dict with the current state of the system
+
+
+        RETURN DICT
+        :orders:        a pandas data frame showing current orders
+        :orderuis:      a list of CarbonOrderUI objects presenting current orders
+        :trades:        a pandas data frame showing trades
         """
         try:
 
@@ -904,7 +1011,8 @@ class CarbonSimulatorUI:
                     ]
                 )
                 orderuis = {o.id: CarbonOrderUI.from_order(o) for o in applicable_orders}
-
+                for o in applicable_orders:
+                    orderuis[o.id].set_linked(orderuis[o.linked_to_id])
             else:
                 orders = pd.DataFrame({})
                 orderuis = dict()
