@@ -22,6 +22,7 @@ print("{0.__name__} v{0.__VERSION__} ({0.__DATE__})".format(CarbonOrderUI))
 
 
 
+
 # ------------------------------------------------------------
 # Test      044
 # File      test_044_MarginalPriceRouting.py
@@ -275,11 +276,15 @@ def test_carbonorderui_tests_dyfromdx_f_and_dxfromdy_f():
     # ### yfromx_f
     
     assert order.yfromx_f(0) == order.yint
-    assert order.yfromx_f(order.xint) == 0
+    assert round(order.yfromx_f(order.xint),5) == 0
     for i in range(10):
         #print(i)
         assert round(order.yfromx_f(order.xfromy_f(i)) - i, 10)  == 0
     
+    order.yfromx_f(order.xint)
+    
+    
+    
     assert orderr.yfromx_f(0) == orderr.yint
     assert round(orderr.yfromx_f(orderr.xint-0.00000001),4)==0
     for i in range(1,10):
@@ -302,6 +307,137 @@ def test_carbonorderui_tests_dyfromdx_f_and_dxfromdy_f():
     assert round(orderr.p_eff_f(0.1)-1000,2) == 0
     assert round(orderr.p_eff_f(10000) - orderr.p0, 6) == 0
     
+    # ### from_Qxy 
+    
+    assert order.Q == sqrt(order.pb_raw/order.pa_raw)
+    assert order.Gamma == 1 - sqrt(order.Q)
+    print(f"Q={order.Q}, Gamma={order.Gamma}")
+    
+    assert order1.Q == sqrt(order1.pb_raw/order1.pa_raw)
+    assert order1.Gamma == 1 - sqrt(order1.Q)
+    print(f"Q={order1.Q}, Gamma={order1.Gamma}")
+    
+    assert orderr.Q == sqrt(orderr.pb_raw/orderr.pa_raw)
+    assert orderr.Gamma == 1 - sqrt(orderr.Q)
+    print(f"Q={orderr.Q}, Gamma={orderr.Gamma}")
+    
+    order_ = CarbonOrderUI.from_Qxy(order.pair, order.tkn, order.Q, order.xint, order.yint, order.y)
+    assert round(order_.B - order.B, 10) == 0
+    assert round(order_.S - order.S, 10) == 0
+    assert round(order_.xint - order.xint, 10) == 0
+    assert round(order_.yint - order.yint, 10) == 0
+    assert round(order_.y - order.y, 10) == 0
+    order_
+    
+    orderr_ = CarbonOrderUI.from_Qxy(orderr.pair, orderr.tkn, orderr.Q, orderr.xint, orderr.yint, orderr.y)
+    assert round(orderr_.B - orderr.B, 10) == 0
+    assert round(orderr_.S - orderr.S, 10) == 0
+    assert round(orderr_.xint - orderr.xint, 10) == 0
+    assert round(orderr_.yint - orderr.yint, 10) == 0
+    assert round(orderr_.y - orderr.y, 10) == 0
+    orderr_
+    
+    # ### buyx
+    
+    order_ = CarbonOrderUI.from_order(order)
+    r = order_.buyx(1000, raiseonerror=True)
+    r
+    
+    assert round(r["x"],10) == 1000
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert round(r["dx/dy"] * r["dy/dx"],10) == 1
+    assert r["tx"] == 'Sell 0.4954541237152398 ETH buy USDC'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)==0
+    
+    r  = order_.buyx(1000, raiseonerror=True)
+    r
+    
+    assert round(r["x"],10) == 2000
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert r["dx/dy"] == 1 / r["dy/dx"]
+    assert r["tx"] == 'Sell 0.4865262015696128 ETH buy USDC'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)!=0
+    
+    order_ = CarbonOrderUI.from_order(orderr)
+    r  = order_.buyx(1, raiseonerror=True)
+    r
+    
+    assert round(r["x"],10) == 1
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert r["dx/dy"] == 1 / r["dy/dx"]
+    assert r["tx"] == 'Sell 986.7796593583731 USDC buy ETH'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)==0
+    
+    r  = order_.buyx(-0.5, raiseonerror=True)
+    r
+    
+    assert round(r["x"],10) == 0.5
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert r["dx/dy"] == 1 / r["dy/dx"]
+    assert r["tx"] == 'Buy 490.10673706911854 USDC sell ETH'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)!=0
+    
+    # ### selly
+    
+    order_ = CarbonOrderUI.from_order(order)
+    r  = order_.selly(1, raiseonerror=True)
+    r
+    
+    assert r["y"] == 9
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert r["dx/dy"] == 1 / r["dy/dx"]
+    assert r["tx"] == 'Sell 1 ETH buy USDC'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)==0
+    
+    r  = order_.selly(1, raiseonerror=True)
+    r
+    
+    assert r["y"] == 8
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert r["dx/dy"] == 1 / r["dy/dx"]
+    assert r["tx"] == 'Sell 1 ETH buy USDC'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)!=0
+    
+    order_ = CarbonOrderUI.from_order(order)
+    r1 = order.selly(1, execute = False)
+    r2 = order.selly(1, execute = False)
+    assert r1 == r2
+    
+    order_ = CarbonOrderUI.from_order(orderr)
+    r  = order_.selly(1000, raiseonerror=True)
+    r
+    
+    assert r["y"] == 9000
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert r["dx/dy"] == 1 / r["dy/dx"]
+    assert r["tx"] == 'Sell 1000 USDC buy ETH'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)==0
+    
+    r  = order_.selly(-500, raiseonerror=True)
+    r
+    
+    assert r["y"] == 9500
+    assert round(r["p"] - sqrt(r["pmarg_old"]*r["pmarg"]),10) == 0
+    assert r["dx/dy"] == 1 / r["dy/dx"]
+    assert r["tx"] == 'Buy 500 USDC sell ETH'
+    assert r["expanded"] == False
+    assert r["y_old"] - r["dy"] == r["y"]
+    assert round(r["x"] - r["dx"],10)!=0
+    
 
 # ------------------------------------------------------------
 # Test      044
@@ -485,7 +621,7 @@ def notest_example_usage():
     
     # And finally we can verify this against the exact algo match
     
-    Sim = CarbonSimulatorUI(pair="ETH/USDC", verbose=False, raiseonerror=False)
+    Sim = CarbonSimulatorUI(pair="ETH/USDC", verbose=False, raiseonerror=True)
     Sim
     
     for i in range(10):
@@ -494,6 +630,8 @@ def notest_example_usage():
         Sim.amm_sells('ETH', 5, use_positions=[i*2])
     Sim.state()['orders'].query("disabled==False")
     
+    Sim.state()
+    
     r = Sim.amm_sells('ETH', 15)['trades']
     r
     
